@@ -1,8 +1,8 @@
 #ifndef _AUDIO_H
 #define _AUDIO_H
 
-#include <stdint.h>
 #include <libconfig.h>
+#include <stdint.h>
 
 typedef struct {
   double current_volume_dB;
@@ -19,7 +19,7 @@ typedef struct {
   // at end of program
   void (*deinit)(void);
 
-  void (*start)(int sample_rate);
+  void (*start)(int sample_rate, int sample_format);
 
   // block of samples
   void (*play)(short buf[], int samples);
@@ -32,17 +32,17 @@ typedef struct {
   // almost certainly wrong if the buffer is empty, so put silent buffers into it to make it busy.
   // will change dynamically, so keep watching it. Implemented in ALSA only.
   // returns a negative error code if there's a problem
-  int (*delay)(long* the_delay); // snd_pcm_sframes_t is a signed long
+  int (*delay)(long *the_delay); // snd_pcm_sframes_t is a signed long
 
   // may be NULL, in which case soft volume is applied
   void (*volume)(double vol);
 
   // may be NULL, in which case soft volume parameters are used
   void (*parameters)(audio_parameters *info);
-  
+
   // may be NULL, in which case software muting is used.
   void (*mute)(int do_mute);
-  
+
 } audio_output;
 
 audio_output *audio_get_output(char *name);
