@@ -24,12 +24,12 @@
  * OTHER DEALINGS IN THE SOFTWARE.
  */
 
+#include "audio.h"
+#include "common.h"
+#include <ao/ao.h>
+#include <memory.h>
 #include <stdio.h>
 #include <unistd.h>
-#include <memory.h>
-#include <ao/ao.h>
-#include "common.h"
-#include "audio.h"
 
 ao_device *dev = NULL;
 
@@ -43,38 +43,73 @@ static void help(void) {
 static int init(int argc, char **argv) {
   const char *str;
   int value;
+  double dvalue;
   ao_initialize();
   int driver = ao_default_driver_id();
   ao_option *ao_opts = NULL;
 
-  config.audio_backend_buffer_desired_length = 44100; // one second.
+  config.audio_backend_buffer_desired_length = 1.0;
   config.audio_backend_latency_offset = 0;
-  
+
   // get settings from settings file first, allow them to be overridden by command line options
 
   if (config.cfg != NULL) {
     /* Get the desired buffer size setting. */
     if (config_lookup_int(config.cfg, "ao.audio_backend_buffer_desired_length", &value)) {
-      if ((value < 0) || (value > 66150))
-        die("Invalid a0 audio backend buffer desired length \"%d\". It should be between 0 and "
-            "66150, default is 44100",
+      if ((value < 0) || (value > 66150)) {
+        inform("The setting ao.audio_backend_buffer_desired_length is deprecated. "
+               "Use ao.audio_backend_buffer_desired_length_in_seconds instead.");
+        die("Invalid ao audio backend buffer desired length \"%d\". It "
+            "should be between 0 and "
+            "66150, default is 6615",
             value);
-      else {
-        config.audio_backend_buffer_desired_length = value;
+      } else {
+        inform("The setting ao.audio_backend_buffer_desired_length is deprecated. "
+               "Use ao.audio_backend_buffer_desired_length_in_seconds instead.");
+        config.audio_backend_buffer_desired_length = 1.0 * value / 44100;
       }
     }
-    
+
+    /* Get the desired buffer size setting. */
+    if (config_lookup_float(config.cfg, "ao.audio_backend_buffer_desired_length_in_seconds",
+                            &dvalue)) {
+      if ((dvalue < 0) || (dvalue > 1.5)) {
+        die("Invalid ao audio backend buffer desired time \"%f\". It "
+            "should be between 0 and "
+            "1.5, default is 0.15 seconds",
+            dvalue);
+      } else {
+        config.audio_backend_buffer_desired_length = dvalue;
+      }
+    }
+
     /* Get the latency offset. */
     if (config_lookup_int(config.cfg, "ao.audio_backend_latency_offset", &value)) {
-      if ((value < -66150) || (value > 66150))
-        die("Invalid ao audio backend buffer latency offset \"%d\". It should be between -66150 and +66150, default is 0",
+      if ((value < -66150) || (value > 66150)) {
+        inform("The setting ao.audio_backend_latency_offset is deprecated. "
+               "Use ao.audio_backend_latency_offset_in_seconds instead.");
+        die("Invalid ao audio backend buffer latency offset \"%d\". It "
+            "should be between -66150 and +66150, default is 0",
             value);
-      else
-        config.audio_backend_latency_offset = value;
+      } else {
+        inform("The setting ao.audio_backend_latency_offset is deprecated. "
+               "Use ao.audio_backend_latency_offset_in_seconds instead.");
+        config.audio_backend_latency_offset = 1.0 * value / 44100;
+      }
+    }
+
+    /* Get the latency offset. */
+    if (config_lookup_float(config.cfg, "ao.audio_backend_latency_offset_in_seconds", &dvalue)) {
+      if ((dvalue < -1.0) || (dvalue > 1.5)) {
+        die("Invalid ao audio backend buffer latency offset time \"%f\". It "
+            "should be between -1.0 and +1.5, default is 0 seconds",
+            dvalue);
+      } else {
+        config.audio_backend_latency_offset = dvalue;
+      }
     }
   }
 
-
   optind = 1; // optind=0 is equivalent to optind=1 plus special behaviour
   argv--;     // so we shift the arguments to satisfy getopt()
   argc++;
@@ -134,9 +169,8 @@ static void deinit(void) {
   ao_shutdown();
 }
 
-static void start(int sample_rate) {
-  if (sample_rate != 44100)
-    die("unexpected sample rate!");
+static void start(int sample_rate, int sample_format) {
+  // this information is not used
 }
 
 static void play(short buf[], int samples) { ao_play(dev, (char *)buf, samples * 4); }
