@@ -28,22 +28,22 @@
  * OTHER DEALINGS IN THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <unistd.h>
+#include "audio.h"
+#include "common.h"
+#include <errno.h>
 #include <fcntl.h>
 #include <memory.h>
+#include <stdio.h>
 #include <stdlib.h>
-#include <errno.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include "common.h"
-#include "audio.h"
+#include <sys/types.h>
+#include <unistd.h>
 
 static int fd = -1;
 
 char *pipename = NULL;
 
-static void start(int sample_rate) {
+static void start(int sample_rate, int sample_format) {
   // this will leave fd as -1 if a reader hasn't been attached
   fd = open(pipename, O_WRONLY | O_NONBLOCK);
 }
@@ -51,7 +51,7 @@ static void start(int sample_rate) {
 static void play(short buf[], int samples) {
   // if the file is not open, try to open it.
   if (fd == -1) {
-     fd = open(pipename, O_WRONLY | O_NONBLOCK); 
+    fd = open(pipename, O_WRONLY | O_NONBLOCK);
   }
   // if it's got a reader, write to it.
   if (fd != -1) {
@@ -60,17 +60,18 @@ static void play(short buf[], int samples) {
 }
 
 static void stop(void) {
-// Don't close the pipe just because a play session has stopped.
-//  if (fd > 0)
-//    close(fd);
+  // Don't close the pipe just because a play session has stopped.
+  //  if (fd > 0)
+  //    close(fd);
 }
 
 static int init(int argc, char **argv) {
   debug(1, "pipe init");
   const char *str;
   int value;
-  
-  config.audio_backend_buffer_desired_length = 44100; // one second.
+  double dvalue;
+
+  config.audio_backend_buffer_desired_length = 1.0;
   config.audio_backend_latency_offset = 0;
 
   if (config.cfg != NULL) {
@@ -79,36 +80,71 @@ static int init(int argc, char **argv) {
     if (config_lookup_string(config.cfg, "pipe.name", &str)) {
       pipename = (char *)str;
     }
- 
+
     if ((pipename) && (strcasecmp(pipename, "STDOUT") == 0))
       die("Can't use \"pipe\" backend for STDOUT. Use the \"stdout\" backend instead.");
 
     /* Get the desired buffer size setting. */
     if (config_lookup_int(config.cfg, "pipe.audio_backend_buffer_desired_length", &value)) {
-      if ((value < 0) || (value > 132300))
-        die("Invalid pipe audio backend buffer desired length \"%d\". It should be between 0 and 132300, default is 44100",
+      if ((value < 0) || (value > 66150)) {
+        inform("The setting pipe.audio_backend_buffer_desired_length is deprecated. "
+               "Use pipe.audio_backend_buffer_desired_length_in_seconds instead.");
+        die("Invalid pipe audio backend buffer desired length \"%d\". It "
+            "should be between 0 and "
+            "66150, default is 6615",
             value);
-      else
-        config.audio_backend_buffer_desired_length = value;
+      } else {
+        inform("The setting pipe.audio_backend_buffer_desired_length is deprecated. "
+               "Use pipe.audio_backend_buffer_desired_length_in_seconds instead.");
+        config.audio_backend_buffer_desired_length = 1.0 * value / 44100;
+      }
+    }
+
+    /* Get the desired buffer size setting. */
+    if (config_lookup_float(config.cfg, "pipe.audio_backend_buffer_desired_length_in_seconds",
+                            &dvalue)) {
+      if ((dvalue < 0) || (dvalue > 1.5)) {
+        die("Invalid pipe audio backend buffer desired time \"%f\". It "
+            "should be between 0 and "
+            "1.5, default is 0.15 seconds",
+            dvalue);
+      } else {
+        config.audio_backend_buffer_desired_length = dvalue;
+      }
     }
 
     /* Get the latency offset. */
     if (config_lookup_int(config.cfg, "pipe.audio_backend_latency_offset", &value)) {
-      if ((value < -66150) || (value > 66150))
-        die("Invalid pipe audio backend buffer latency offset \"%d\". It should be between -66150 and +66150, default is 0",
+      if ((value < -66150) || (value > 66150)) {
+        inform("The setting pipe.audio_backend_latency_offset is deprecated. "
+               "Use pipe.audio_backend_latency_offset_in_seconds instead.");
+        die("Invalid pipe audio backend buffer latency offset \"%d\". It "
+            "should be between -66150 and +66150, default is 0",
             value);
-      else
-        config.audio_backend_latency_offset = value;
+      } else {
+        inform("The setting pipe.audio_backend_latency_offset is deprecated. "
+               "Use pipe.audio_backend_latency_offset_in_seconds instead.");
+        config.audio_backend_latency_offset = 1.0 * value / 44100;
+      }
+    }
+
+    /* Get the latency offset. */
+    if (config_lookup_float(config.cfg, "pipe.audio_backend_latency_offset_in_seconds", &dvalue)) {
+      if ((dvalue < -1.0) || (dvalue > 1.5)) {
+        die("Invalid pipe audio backend buffer latency offset time \"%f\". It "
+            "should be between -1.0 and +1.5, default is 0 seconds",
+            dvalue);
+      } else {
+        config.audio_backend_latency_offset = dvalue;
+      }
     }
   }
-  
   if ((pipename == NULL) && (argc != 1))
     die("bad or missing argument(s) to pipe");
 
   if (argc == 1)
     pipename = strdup(argv[0]);
-  
-  
+
   // here, create the pipe
   if (mkfifo(pipename, 0644) && errno != EEXIST)
     die("Could not create output pipe \"%s\"", pipename);
@@ -119,13 +155,11 @@ static int init(int argc, char **argv) {
 }
 
 static void deinit(void) {
-   if (fd > 0)
+  if (fd > 0)
     close(fd);
 }
 
-static void help(void) {
-  printf("    pipe takes 1 argument: the name of the FIFO to write to.\n");
-}
+static void help(void) { printf("    pipe takes 1 argument: the name of the FIFO to write to.\n"); }
 
 audio_output audio_pipe = {.name = "pipe",
                            .help = &help,
